/// <reference types="node" />
import { Serializable } from 'child_process';
import NodeCache, { Options, Key } from './node_cache';
export type RefreshOptions = Options & {
    /** Time to refresh in seconds. If not prowided: 70% of defaultTTL if defaultTTL = 0 ttr = 10 minutes. */
    ttr?: number;
    /** To refresh again till ttl. */
    retryOnError?: boolean;
    /** Pause between retry on error. */
    retryPause?: number;
    /** Will store clones of arguments. */
    argsUseClones?: boolean;
};
/**
 * A are the arguments to the refreshMethod method refreshing keys
 * T is a type of the value to be cached
 */
export declare class NodeCacheTs<A extends {
    [key: string]: Serializable;
}, T> extends NodeCache<T> {
    refreshMethod: (args: A) => Promise<T>;
    private _refreshArgsCache;
    private _runningCalls;
    ttr: number;
    retryPause: number;
    ttrAsFractionOfTTL: number;
    defaultTTR: number;
    refreshOptions: RefreshOptions;
    constructor(options: RefreshOptions | undefined, refreshMethod: (args: A) => Promise<T>);
    call(key: Key, args: A): Promise<T>;
    flushAll(): void;
    del(keys: Key | Key[]): number;
    private refreshEntry;
}
