"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeCacheTs = void 0;
const node_cache_1 = __importDefault(require("./node_cache"));
/**
 * A are the arguments to the refreshMethod method refreshing keys
 * T is a type of the value to be cached
 */
class NodeCacheTs extends node_cache_1.default {
    refreshMethod;
    _refreshArgsCache;
    _runningCalls = {};
    ttr;
    retryPause;
    ttrAsFractionOfTTL = 0.7;
    //default ttr if no 
    defaultTTR = 600;
    refreshOptions = {};
    constructor(options = {}, refreshMethod) {
        super(options);
        this.refreshOptions = options;
        this.ttr = options.ttr || (options.stdTTL ? (options.stdTTL * this.ttrAsFractionOfTTL) : this.defaultTTR);
        // if error ocured wait given time or halve of time to refresh before next refresh
        this.retryPause = options.retryPause || Math.ceil(this.ttr / 5);
        this._refreshArgsCache = new node_cache_1.default({ stdTTL: this.ttr, checkperiod: options.checkperiod, useClones: this.refreshOptions.argsUseClones }).on('expired', (key, value) => {
            this.refreshEntry(key, value);
        });
        this.refreshMethod = refreshMethod;
    }
    async call(key, args) {
        let response;
        if (super.has(key)) {
            response = super.get(key);
        }
        else {
            const waitingCall = this._runningCalls[key];
            if (waitingCall) {
                response = await waitingCall;
            }
            else {
                try {
                    const promise = this.refreshMethod(args);
                    this._runningCalls[key] = promise;
                    response = await promise;
                    super.set(key, response);
                }
                finally {
                    delete this._runningCalls[key];
                }
            }
        }
        // if no refresh sheduled schedule one
        if (!this._refreshArgsCache.has(key)) {
            this._refreshArgsCache.set(key, args);
        }
        return await Promise.resolve(response);
    }
    flushAll() {
        super.flushAll();
        this._refreshArgsCache.flushAll();
    }
    del(keys) {
        this._refreshArgsCache.del(keys);
        return super.del(keys);
    }
    async refreshEntry(key, args) {
        try {
            const response = await this.refreshMethod(args);
            super.set(key, response);
        }
        catch (error) {
            const ttl = super.getTtl(key);
            if (ttl && this.retryPause * 1000 < ttl) {
                this._refreshArgsCache.set(key, args, this.retryPause);
            }
            this.emit('refresh_error', error, key, args);
        }
    }
}
exports.NodeCacheTs = NodeCacheTs;
//# sourceMappingURL=node_cache_ts.js.map