"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = __importDefault(require("http"));
const node_cache_1 = __importDefault(require("../src/node_cache"));
const node_cache_ts_1 = require("../src/node_cache_ts");
const promises_1 = require("timers/promises");
const usage = `
Use it like this, where 12 is a number of miliseconds to sleep:
http//localhost:7878/{raw,node,snow}/{sleep_time}/{key}/{value}/{error probability 0-1}
`;
const nodeCache = new node_cache_1.default({
    stdTTL: 20
});
const snowCache = new node_cache_ts_1.NodeCacheTs({
    stdTTL: 20,
    ttr: 0.01
}, async (args) => {
    return await mockServiceCall(args.sleepTime, args.key, args.value, args.flak);
});
const mockServiceCall = async (sleepTime, key, value, flak) => {
    await (0, promises_1.setTimeout)(sleepTime);
    const answer = `{
            "now":${Date.now()},
            "key":${key},
            "value":${value}
    }\n`;
    if (Math.random() < flak) {
        throw Error('Random error');
    }
    return answer;
};
const call = async (cacheType, sleepTime, key, value, flak) => {
    if (cacheType === 'node') {
        const cachedAnswer = nodeCache.get(key);
        if (cachedAnswer) {
            return cachedAnswer;
        }
        else {
            await (0, promises_1.setTimeout)(sleepTime);
            const answer = await mockServiceCall(sleepTime, key, value, flak);
            nodeCache.set(key, answer);
            return answer;
        }
    }
    else if (cacheType === 'snow') {
        return snowCache.call(key, { sleepTime, key, value, flak });
    }
    else {
        return mockServiceCall(sleepTime, key, value, flak);
    }
};
http_1.default.createServer(async (req, res) => {
    if (req.url === '/') {
        res.writeHead(200, { 'Content-Type': 'text/html' });
        res.write('Hello to mock server!\n');
        res.write(usage);
        res.end();
    }
    else if ((req.url?.startsWith('/sleep/') || req.url?.startsWith('/node/') || req.url?.startsWith('/snow/')) && req.url.split('/')[2]) {
        const cacheType = req.url.split('/')[1] || 'sleep';
        const sleepTime = Number(req.url.split('/')[2]);
        const key = req.url.split('/')[3] || 'NaN';
        const value = req.url.split('/')[4] || 'NaN';
        const flak = Number(req.url.split('/')[5]);
        try {
            const answer = await call(cacheType, sleepTime, key, value, flak);
            res.writeHead(200, { 'Content-Type': 'text/html' });
            res.write(answer);
            res.end();
        }
        catch (err) {
            res.writeHead(500, { 'Content-Type': 'text/html' });
            res.end();
        }
    }
    else {
        res.writeHead(404, { 'Content-Type': 'text/html' });
        res.write("Goto main page '/'!\n");
        res.write(usage);
        res.end();
    }
}).listen(7878);
//# sourceMappingURL=mock_server.js.map